﻿package
{
	import flash.display.DisplayObject;
	import flash.display.Sprite;
	import flash.events.Event;
	import flash.events.FullScreenEvent;
	import flash.events.MouseEvent;
	import flash.geom.Point;
	
	import com.greensock.*;
	import com.greensock.easing.*;
	
	import net.delaunay.*;
	
	public class Boussoler extends Sprite
	{
		private var nodes:Array = new Array();
		private var triangleP:Vector.<Array> = new Vector.<Array>();
		private var numP:int = 160;
		private var upToStage:int = 200;
		private var minDist:int = 10;
		private var l:Sprite;
		private var delaunay:Array = new Array();
		
		public function Boussoler()
		{
			addEventListener(Event.ADDED_TO_STAGE, addedHandler);
		}
		
		private function fullhandler(e:FullScreenEvent):void
		{
			init();
		}
		
		private function downHandler(e:MouseEvent):void
		{	
			removeEventListener(Event.ENTER_FRAME, render);
			nodes = [];
			delaunay = [];
			triangleP = new Vector.<Array>;
			init();
		}
		
		private function addedHandler(e:Event):void
		{
			stage.addEventListener(MouseEvent.MOUSE_DOWN, downHandler);
			stage.addEventListener(FullScreenEvent.FULL_SCREEN,fullhandler);
			init();
		}
		
		private function init():void
		{
			
			var pt:DelaunayPoint;
			
			for (var i:int = 0; i < numP; i++){
				
				var setX:int = (Math.random()*(stage.stageWidth+upToStage))-upToStage/2;
				var setY:int = (Math.random()*(stage.stageHeight+upToStage))-upToStage/2;
			
				for (var j:int = 0; j < i; j++){
					var distance = getDist(nodes[j].x, nodes[j].y, setX, setY);
					
					if (distance < minDist){
						setX = (Math.random()*(stage.stageWidth+upToStage))-upToStage/2;
						setY = (Math.random()*(stage.stageHeight+upToStage))-upToStage/2;
					}
				}
				
				// Each point becomes a DelaunayPoint(class) for the calculation
				// it is inserted in nodes
				pt = new DelaunayPoint( setX, setY );
				nodes.push(pt);
			}
			
			// calculates the triangulation according to the nodes
			delaunay = Delaunay.Triangulate(nodes);
			
			addEventListener(Event.ENTER_FRAME, render);
		}
		
		private function render(e:Event):void
		{
			graphics.clear();
			
			var t:Triangle;
			var i:int;
			var L:int = delaunay.length;
			
			var center:Point;
			
			// each triangle is drawn and calculate his proximity to the mouse
			// the moves the points of that triangle
			for ( i = 0; i < L; i++ ){
				
				t = ( delaunay[ i ] as Triangle );
				
				var p1:Point = new Point(t.p1.x, t.p1.y);
				var p2:Point = new Point(t.p2.x, t.p2.y);
				var p3:Point = new Point(t.p3.x, t.p3.y);
								
				triangleP.push(new Array(p1, p2, p3));
				
				//graphics.lineStyle( 1, 0x006699, .5);
				graphics.beginFill( 0xffffff );
				graphics.moveTo(triangleP[i][0].x, triangleP[i][0].y);
				graphics.lineTo(triangleP[i][1].x, triangleP[i][1].y);
				graphics.lineTo(triangleP[i][2].x, triangleP[i][2].y);
				graphics.lineTo(triangleP[i][0].x, triangleP[i][0].y);
				graphics.endFill();

				center = t.getCenter();
				
				var mouseP:Point = new Point(mouseX, mouseY);
				
				// check distance between mouse and gravityCenter of triangles
				var centerDist:int = getDist(mouseP.x, mouseP.y, center.x, center.y);
				
				// if the mouse is "near" the triangle it checks the nearest point of that 
				// triangle and moves it to the center of the opposit hedge
				if (centerDist < 100){
					
					var pDist1:int =  getDist(mouseP.x, mouseP.y, triangleP[i][0].x, triangleP[i][0].y);
					var pDist2:int =  getDist(mouseP.x, mouseP.y, triangleP[i][1].x, triangleP[i][1].y);
					var pDist3:int =  getDist(mouseP.x, mouseP.y, triangleP[i][2].x, triangleP[i][2].y);
					
					if(pDist1 < pDist2 && pDist1 < pDist3){
						var arr1:Point = getSideCenter(triangleP[i][1].x, triangleP[i][1].y, triangleP[i][2].x, triangleP[i][2].y);
						TweenLite.to(triangleP[i][0], .5, {x: arr1.x, y:arr1.y, ease:Linear.easeNone});
					}
					
					if(pDist2 < pDist1 && pDist2 < pDist3){
						var arr2:Point = getSideCenter(triangleP[i][0].x, triangleP[i][0].y, triangleP[i][2].x, triangleP[i][2].y);
						TweenLite.to(triangleP[i][1], .5, {x: arr2.x, y:arr2.y, ease:Linear.easeNone});
					}
					
					if(pDist3 < pDist1 && pDist3 < pDist2){
						var arr3:Point = getSideCenter(triangleP[i][1].x, triangleP[i][1].y, triangleP[i][0].x, triangleP[i][0].y);
						TweenLite.to(triangleP[i][2], .5, {x: arr3.x, y:arr3.y, ease:Linear.easeNone});
					}
				} else {
					// points return to their original place!
					TweenLite.to(triangleP[i][0], 1, {x:t.p1.x, y:t.p1.y, ease:Linear.easeNone});
					TweenLite.to(triangleP[i][1], 1, {x:t.p2.x, y:t.p2.y, ease:Linear.easeNone});
					TweenLite.to(triangleP[i][2], 1, {x:t.p3.x, y:t.p3.y, ease:Linear.easeNone});
				}
				
				/*graphics.lineStyle( 1, 0xffffff, .7);
				graphics.drawCircle(t.p1.x, t.p1.y, 5);
				graphics.drawCircle(t.p2.x, t.p2.y, 5);
				graphics.drawCircle(t.p3.x, t.p3.y, 5);
				*/	
			}
		}
		
		function getSideCenter(x1:Number, y1:Number, x2:Number, y2:Number):Point {
			
			var mid:Point = new Point();
			mid.x = x1 + ( x2 - x1 )/2;
			mid.y = y1 + ( y2 - y1 )/2;
			
			return new Point(mid.x, mid.y);
		}
		
		function getDist(x1:Number, y1:Number, x2:Number, y2:Number):Number {
			var dx:Number = x1 - x2;
			var dy:Number = y1 - y2;
			var dist:int = Math.sqrt(Math.pow(dx, 2) + Math.pow(dy, 2));
			return dist;
		}
	}
}